package solutions.reslate.entertainment.spigotresourcesync;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import solutions.reslate.entertainment.spigotresourcesync.data.Configuration;
import solutions.reslate.entertainment.spigotresourcesync.data.SyncPair;
import solutions.reslate.entertainment.spigotresourcesync.events.ObjectLoadListener;
import solutions.reslate.entertainment.spigotresourcesync.synchronisation.Synchroniser;

public class SyncListManager implements ObjectLoadListener<Configuration> {
    private Configuration configuration;
    private Synchroniser synchroniser;
    private Logger logger;
    
    public SyncListManager(Configuration configuration, Synchroniser synchroniser, Logger logger) {
        super();
        this.logger = logger;
        this.configuration = configuration;
        this.synchroniser = synchroniser;
    }

    public void synchroniseAllSyncList() {
        logger.info("Synchronising all sync pairs...");
        for (SyncPair syncPair : this.configuration.getSyncList().gatherAllSyncPairs()) {
            File source = new File(syncPair.getSource());
            File dest = new File(syncPair.getDestination());
            try {
                synchroniser.sync(source, dest);
                logger.info(String.format("Synchronised \"%s\" to \"%s\"!", source.getName(), dest.getName()));
            } catch (IOException e) {
                logger.warning(String.format("Failed to synchronise \"%s\" to \"%s\". %s", source.getAbsolutePath(), dest.getAbsolutePath(), e.getMessage()));
            }
        }
        logger.info("Done synchronising.");
    }

    public void synchroniseGroupSyncList(String group) {
        logger.info("Synchronising group sync pairs...");
        for (SyncPair syncPair : this.configuration.getSyncList().gatherGroupSyncPairs(group)) {
            File source = new File(syncPair.getSource());
            File dest = new File(syncPair.getDestination());
            try {
                synchroniser.sync(source, dest);
                logger.info(String.format("Synchronised \"%s\" to \"%s\"!", source.getName(), dest.getName()));
            } catch (IOException e) {
                logger.warning(String.format("Failed to synchronise \"%s\" to \"%s\". %s", source.getAbsolutePath(), dest.getAbsolutePath(), e.getMessage()));
            }
        }
        logger.info("Done synchronising.");
    }

    @Override
    public void objectLoaded(Configuration obj) {
        logger.info("Updating synchronisation list due to recently loading configuration...");
        this.configuration = obj;
        synchroniseAllSyncList();
        logger.info("Done.");
    }

}
