package solutions.reslate.entertainment.spigotresourcesync.data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.SequencedCollection;
import java.util.SequencedMap;


public class SyncList implements Serializable {
    private SequencedMap<String, SequencedCollection<SyncPair>> syncPairs;

    public SyncList() {
        super();
        this.syncPairs = new LinkedHashMap<>();
    }

    public SequencedCollection<SyncPair> gatherAllSyncPairs() {
        List<SyncPair> allSyncPairs = new ArrayList<>();
        for (SequencedCollection<SyncPair> syncPairs : this.syncPairs.values()) {
            allSyncPairs.addAll(syncPairs);
        }
        return Collections.unmodifiableSequencedCollection(allSyncPairs);
    }

    public SequencedCollection<SyncPair> gatherGroupSyncPairs(String groupName) {
        if (!this.syncPairs.containsKey(groupName)) throw new IllegalArgumentException(String.format("\"%s\" group does not exist!", groupName));
        return Collections.unmodifiableSequencedCollection(this.syncPairs.get(groupName));
    }

    public void addSynchronisationPair(String source, String dest, String group) {
        this.addSynchronisationPair(new SyncPair(source, dest), group);
    }

    public void addSynchronisationPair(SyncPair syncPair, String group) {
        addSynchronisationGroup(group);
        this.syncPairs.get(group).add(syncPair);
    }

    public void removeSynchronisationSet(String source, String dest, String group) {
        if (this.syncPairs.containsKey(group)) {
            this.syncPairs.get(group).remove(new SyncPair(source, dest));
        }
    }

    public void addSynchronisationGroup(String group) {
        if (!this.syncPairs.containsKey(group)) {
            this.syncPairs.put(group, new LinkedHashSet<>());
        }
    }

    public void removeSynchronisationGroup(String group) {
        if (this.syncPairs.containsKey(group)) {
            this.syncPairs.remove(group);
        }
    }

    public SequencedCollection<String> gatherSynchronisationGroups() {
        return Collections.unmodifiableSequencedCollection(new ArrayList<>(this.syncPairs.keySet()));
    }

    public SequencedMap<String, SequencedCollection<SyncPair>> getSyncPairs() {
        return syncPairs;
    }

    public void setSyncPairs(SequencedMap<String, SequencedCollection<SyncPair>> syncPairs) {
        this.syncPairs = syncPairs;
    }

}
