package solutions.reslate.entertainment.spigotresourcesync.serialisation;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;
import com.fasterxml.jackson.dataformat.yaml.YAMLGenerator.Feature;

public class JacksonYamlSerialiser<T> implements Serialiser<T> {

    private ObjectMapper mapper;

    public JacksonYamlSerialiser() {
        super();
        YAMLFactory yamlFactory = new YAMLFactory()
                .disable(Feature.WRITE_DOC_START_MARKER);
        mapper = new ObjectMapper(yamlFactory)
                .disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);
    }

    @Override
    public void flushTo(File file, T serialisable) throws IOException {
        try (FileWriter writer = new FileWriter(file)) {
            mapper.writeValue(writer, serialisable);
        }
    }

    @Override
    public T load(File file, Class<T> type) throws IOException {
        try (FileReader reader = new FileReader(file)) {
            return this.load(reader, type);
        }
    }
    
    @Override
    public T load(Reader reader, Class<T> type) throws IOException {
        try {
            return mapper.readValue(reader, type);
        } catch (IOException e) {
            throw e;
        }
    }

    @Override
    public String serialize(T serialisable) {
        try {
            return mapper.writeValueAsString(serialisable);
        } catch (JsonProcessingException e) {
            throw new IllegalArgumentException("Given type cannot be serialised.", e);
        }
    }

    @Override
    public T deserialize(String data, Class<T> type) {
        try {
            return mapper.readValue(data, type);
        } catch (JsonProcessingException e) {
            throw new IllegalArgumentException("Given string cannot be deserialised.", e);
        }
    }

}
