package solutions.reslate.entertainment.spigotresourcesync.serialisation;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assumptions.assumeTrue;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.SequencedCollection;
import java.util.SequencedMap;

import org.junit.jupiter.api.Test;

import solutions.reslate.entertainment.spigotresourcesync.data.SyncPair;

public class TestJacksonYamlSerialiser {
    @Test
    void testListSerialisation() throws IOException {
        ArrayList<String> dummyList = new ArrayList<>();
        dummyList.add("abc");
        dummyList.add("def");
        File dummyFile = File.createTempFile("dummy_file", ".yml");
        JacksonYamlSerialiser<List<String>> dummyJacksonYamlSerialiser = new JacksonYamlSerialiser<>();
        dummyJacksonYamlSerialiser.flushTo(dummyFile, dummyList);
        assumeTrue(dummyFile.exists());
        List<String> writtenLines = Files.readAllLines(dummyFile.toPath());
        assertEquals(2, writtenLines.size());
        assertEquals("- \"abc\"", writtenLines.get(0));
        assertEquals("- \"def\"", writtenLines.get(1));
    }

    @Test
    void testSyncPairSerialisation() throws IOException {
        File dummyFile = File.createTempFile("dummy_file", ".yml");
        SyncPair dummySyncPair = new SyncPair("abc", "def");
        JacksonYamlSerialiser<SyncPair> dummyJacksonYamlSerialiser = new JacksonYamlSerialiser<>();
        dummyJacksonYamlSerialiser.flushTo(dummyFile, dummySyncPair);
        assumeTrue(dummyFile.exists());
        List<String> writtenLines = Files.readAllLines(dummyFile.toPath());
        assertEquals(2, writtenLines.size());
        assertEquals("source: \"abc\"", writtenLines.get(0));
        assertEquals("destination: \"def\"", writtenLines.get(1));
    }

    @Test
    void testSequencedCollectionSerialisation() throws IOException {
        File dummyFile = File.createTempFile("dummy_file", ".yml");
        SequencedCollection<String> collection = new LinkedHashSet<>();
        collection.add("abc");
        collection.add("def");
        JacksonYamlSerialiser<SequencedCollection<String>> dummyJacksonYamlSerialiser = new JacksonYamlSerialiser<>();
        dummyJacksonYamlSerialiser.flushTo(dummyFile, collection);
        assumeTrue(dummyFile.exists());
        List<String> writtenLines = Files.readAllLines(dummyFile.toPath());
        assertEquals(2, writtenLines.size());
        assertEquals("- \"abc\"", writtenLines.get(0));
        assertEquals("- \"def\"", writtenLines.get(1));
    }

    @Test
    void testSequencedMapSerialisation() throws IOException {
        File dummyFile = File.createTempFile("dummy_file", ".yml");
        SequencedMap<String, String> collection = new LinkedHashMap<>();
        collection.put("abc", "def");
        collection.put("123", "456");
        JacksonYamlSerialiser<SequencedMap<String, String>> dummyJacksonYamlSerialiser = new JacksonYamlSerialiser<>();
        dummyJacksonYamlSerialiser.flushTo(dummyFile, collection);
        assumeTrue(dummyFile.exists());
        List<String> writtenLines = Files.readAllLines(dummyFile.toPath());
        assertEquals(2, writtenLines.size());
        assertEquals("abc: \"def\"", writtenLines.get(0));
        assertEquals("\"123\": \"456\"", writtenLines.get(1));
    }
}
